extends CharacterBody3D

const SPEED = 40.0
const ACCELERATION = 1.5

@onready var CannonArray = [$Cannon, $Cannon2]
@onready var MainScene = get_tree().current_scene
@onready var sound_bullet = $Sound_bullet
@onready var sound_crush = $Sound_Crush

const CrushEffect = preload("res://player_crushed.tscn")
const Bullet = preload("res://Units/bullet.tscn")

var Able_to_shoot = true
var Crushed = false
var direction = Vector3.ZERO
var CrushVector = Vector3.ZERO
var CrushSFX

signal Crush

func _ready():
	setStartPosotion()

func setStartPosotion():
	position = Vector3(0.0, -4.0, -8.0)

func _physics_process(delta):
	if !Crushed:
		velocity.x = move_toward(velocity.x, direction.x*SPEED, ACCELERATION)
		velocity.y = move_toward(velocity.y, direction.y*SPEED, ACCELERATION)
		move_and_slide()
	else:
		velocity.x = move_toward(velocity.x, CrushVector.x, ACCELERATION)
		velocity.y = move_toward(velocity.y, CrushVector.y, ACCELERATION)
		velocity.z = move_toward(velocity.z, CrushVector.z, ACCELERATION)
		move_and_slide()

func _process(delta):
	if !Crushed:
		direction = Input.get_vector("ui_left", "ui_right", "ui_down", "ui_up")
		transform.origin.x = clamp(transform.origin.x, -15, 15)
		transform.origin.y = clamp(transform.origin.y, -10, 10)
		rotation_degrees.z = clamp(velocity.x * -2, -55.0, 55.0)
		rotation_degrees.x = clamp(velocity.y/2, -10.0, 20.0)
		if Input.is_action_pressed("ui_accept"):
			if Able_to_shoot:
				Able_to_shoot = false
				sound_bullet.play()
				for i in CannonArray:
					var bullet = Bullet.instantiate()
					MainScene.add_child(bullet)
					bullet.transform = i.global_transform
					bullet.velocity = bullet.transform.origin.direction_to($Cursor.position3d) * 250
					bullet.look_at_from_position(bullet.transform.origin, $Cursor.position3d)

func _on_fire_cooldown_timeout():
	Able_to_shoot = true

func _on_hit_box_body_entered(body):
	if body.is_in_group("ENEMIES") and !$HitBox/CollisionShape3D.disabled:
		crush()

func crush():
	Crushed = true
	emit_signal("Crush")
	$HitBox/CollisionShape3D.disabled = true
	sound_crush.play()
	$EngineTrail.emitting = false
	if !is_instance_valid(CrushSFX):
		CrushSFX = CrushEffect.instantiate()
		add_child(CrushSFX)
	CrushVector = Vector3(velocity.x, abs(velocity.y)*-1 - 10.0 , -10.0)
	$Respawn.start()

func _on_respawn():
	if !MainScene.GameOver:
		setStartPosotion()
		Crushed = false
		velocity = Vector3.ZERO
		CrushSFX.queue_free()
		$AnimationPlayer.play("Invisibility")
		$EngineTrail.emitting = true
